
#include<avr\io.h>
#include <avr\delay.h>
#include<avr\interrupt.h>
#include<stdlib.h>
#include<math.h>


#define enable_conv()	{ ADCSRA |= (1<<ADEN); }
#define disable_conv()	{ ADCSRA &= ~(1<<ADEN); } 

double F_ref[6];
int F_curent;

//-------------LED FUNCTIONS---------------
void led_init()
	{
		DDRB |= (1<<PB0) | (1<<PB1) | (1<<PB2);
	}

void correct_led()
	{
		PORTB |= (1<<PB1);
		PORTB &= ~(1<<PB0) & ~(1<<PB2);
	}
	
void lower_led()
	{
		PORTB |= (1<<PB0);
		PORTB &= ~(1<<PB1) & ~(1<<PB2);
	}

void higher_led()
	{
		PORTB |= (1<<PB2);
		PORTB &= ~(1<<PB0) & ~(1<<PB1);
	}
	
void err_led()
	{
		PORTB |= (1<<PB0) | (1<<PB2);
		PORTB &= ~(1<<PB1);
	}

void off_led()
	{
		PORTB &= ~(1<<PB0) & ~(1<<PB1) & ~(1<<PB2);
	}

//------------BUTTON FUNCTIONS----------------	
void init_but()
	{
	 DDRC &= ~(1<<PC0) & ~(1<<PC1) & ~(1<<PC2) & ~(1<<PC3) & ~(1<<PC4) & ~(1<<PC5);
	 PORTC |= (1<<PC0) | (1<<PC1) | (1<<PC2) | (1<<PC3) | (1<<PC4) | (1<<PC5);
	}

int sel_but()
	{
	 int b_sel = -1;
	 int selected = 0;
	
	 if (!(PINC & (1<<PC0)))
		{
			b_sel = 0;
			selected = 1;
		}
	
	 if (!(PINC & (1<<PC1)))
		{
		  if(selected)
			return -2;
		  else
		  {
			b_sel = 1;
			selected = 1;
		  }
		}
	
	 if (!(PINC & (1<<PC2)))
		{
			if(selected)
				return -2;
			else
			{
				b_sel = 2;
				selected = 1;
			}
		}
		
	if (!(PINC & (1<<PC3)))
		{
			if(selected)
				return -2;
			else
			{
				b_sel = 3;
				selected = 1;
			}
		}
		
	if (!(PINC & (1<<PC4)))
		{
			if(selected)
				return -2;
			else
			{
				b_sel = 4;
				selected = 1;
			}
		}
		
	 if (!(PINC & (1<<PC5)))
		{
			if(selected)
				return -2;
			else
			{
				b_sel = 5;
				selected = 1;
			}
		}
	
	 return b_sel;
	}
	
//-----------ADC FUNCTIONS---------------------
void ADC_init()	
	{
		ADMUX = 0;
		ADMUX |= (1<<REFS0) | (1<<ADLAR);
		ADCSRA = 0;
		ADCSRA |= (1<<ADIE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0) | (1<<ADATE);
		SFIOR &= ~(1<<ADTS0) & ~(1<<ADTS1) & ~(1<<ADTS2);
		ADCSRA |= (1<<ADEN);
	}
	
float ADC_get()
	{
		
		ADCSRA |= (1<<ADSC);
		loop_until_bit_is_set(ADCSRA,ADIF);
		return ADCH;
	}
	
//-----------FOURIER------------------------------ 


void fourier(float *x)
	{
		long i,j,k,logN,le,le2;
		float *p, *q,t, ur, ui,arg,wr,wi,*p1r,*p1i,*p2r,*p2i,tr,ti;
		logN = (long)(log(16)/log(2.) + .5);
		
		
		for(i=2;i< 30;i+=2)
			{
				for(k=2, j=0;k<32;k<<=1)
					{
						if ( i & k ) j++;
						j <<=1;
					}
				if (i < j)
					{
					 p = x + i;
					 q = x + j;
					 t = *p;
					 *(p++) = *q;
					 *(q++) = t;
					 t = *p;
					 *p = *q;
					 *q = t;
					}
			}
			
		for (k = 0, le = 2; k<logN; k++)
			{
				le <<= 1;
				le2 = le >> 1;
				ur = 1.0;
				ui = 0.0;
				arg = M_PI/(le2>>1);
				wr = cos(arg);
				wi = -sin(arg);
				
				for(j=0;j<le2;j+=2)
					{
					 p1r = x + j; p1i = p1r + 1;
					 p2r = p1r + le2; p2i= p2r + 1;
					 for(i=j;i<32;i+=le)
						{
							tr = *p2r * ur - *p2i * ui;
							ti = *p2r * ui + *p2i * ur;
							*p2r = *p1r - tr;
							*p2i = *p1i - ti;
							*p1r += tr;
							*p1i += ti;
							p1r += le;
							p1i += le;
							p2r += le;
							p2i += le;
						}
					tr = ur *wr - ui * wi;
					ui = ur * wi + ui * wr;
					ur = tr;
					}
			}
	}
							
		
		
void init()
	{
	 led_init();
	 init_but();
	 ADC_init();
	 F_ref[0] = 330;
	 F_ref[1] = 247;
	 F_ref[2] = 196;
	 F_ref[3] = 146;
	 F_ref[4] = 110;
	 F_ref[5] = 83;
	 ADC_get();
	 //sei();
	}
	 
int main()
{

init();

int x,i,err = 0;
float *v,max,maxi;
v = (float*) malloc (32 * sizeof(float));
double mean;
int ok,h,l;

while(1)
	{
		err = 0;
		while( (x=sel_but() ) == -1);
		
		for(i=0;i<32;i+=2)
			{
			 v[i] = (float)ADC_get();
			 v[i+1] = 0;
			 if (sel_but() == -2)
					{
						err_led();
						err = 1;
						break;
					}
			}
		
		if(err)
			continue;
		
		fourier(v);
		
		max = -100; maxi = 0;
		
		F_curent = F_ref[x];
		 
		
		
		
		
		for(i=0;i<16;i++)
			if ( v[i] > max )
				{max = v[i]; maxi = i; }
		
		max = maxi * 125000/ 208;
				
		if(max < F_curent +15 && max > F_curent -15)
				correct_led();
		else
			if (max > F_curent)
					higher_led();
			else
					lower_led();
		
		
		
	}

}
